/* Generated by cmake */

#ifndef CPL_CONFIG_H
#define CPL_CONFIG_H

#ifdef _MSC_VER
# ifndef CPL_DISABLE_STDCALL
#  define CPL_STDCALL __stdcall
# endif
#endif

/* --prefix directory for GDAL install */
#define GDAL_PREFIX "/usr"

/* The size of `int', as computed by sizeof. */
#define SIZEOF_INT 4

/* The size of `unsigned long', as computed by sizeof. */
#define SIZEOF_UNSIGNED_LONG 8

/* The size of `void*', as computed by sizeof. */
#define SIZEOF_VOIDP 8

/* The size of `size_t', as computed by sizeof. */
#define SIZEOF_SIZE_T 8

/* Whether `std::float16_t` is available (and working). */
/* #undef HAVE_STD_FLOAT16_T */

/* Whether `_Float16' is supported. */
#define HAVE__FLOAT16 1

/* Define to 1, if you have LARGEFILE64_SOURCE */
/* #undef VSI_NEED_LARGEFILE64_SOURCE */

/* Define to 1 if you want to use the -fvisibility GCC flag */
#define USE_GCC_VISIBILITY_FLAG 1

/* Define to 1 if GCC atomic builtins are available */
#define HAVE_GCC_ATOMIC_BUILTINS 1

/* Define to 1 if your processor stores words with the most significant byte
   first (like Motorola and SPARC, unlike Intel and VAX). */
/* #undef WORDS_BIGENDIAN */

/* Define to name of 64bit stat structure */
#define VSI_STAT64_T stat64


#ifdef GDAL_COMPILATION

/* The size of `long int', as computed by sizeof. */
#define SIZEOF_LONG_INT 8

/* Define if you want to use pthreads based multiprocessing support */
#define CPL_MULTIPROC_PTHREAD 1

/* Define to 1 if you have the `PTHREAD_MUTEX_RECURSIVE' constant. */
#define HAVE_PTHREAD_MUTEX_RECURSIVE 1

/* Define to 1 if you have the `PTHREAD_MUTEX_ADAPTIVE_NP' constant. */
#define HAVE_PTHREAD_MUTEX_ADAPTIVE_NP 1

/* Define to 1 if you have the `pthread_spin_lock' function. */
#define HAVE_PTHREAD_SPIN_LOCK 1

/* Define to 1 if you have the `pthread_atfork' function. */
#define HAVE_PTHREAD_ATFORK 1

/* Define to 1 if you have the 5 args `mremap' function. */
#define HAVE_5ARGS_MREMAP 1

/* Define to 1 if you have the <sys/random.h> header file. */
#define HAVE_SYS_RANDOM_H 1

/* Define to 1 if you have the `getrandom' function. */
#define HAVE_GETRANDOM 1

/* Define to 1 if you have the `getrlimit' function. */
#define HAVE_GETRLIMIT 1

/* Define to 1 if you have the `RLIMIT_AS' constant. */
#define HAVE_RLIMIT_AS 1

/* Define to 1 if you have the `dl_iterate_phdr' function. */
#define HAVE_DL_ITERATE_PHDR 1

/* Define to 1 if you have the <direct.h> header file. */
/* #undef HAVE_DIRECT_H */

/* Define to 1 if you have the <dlfcn.h> header file. */
#define HAVE_DLFCN_H 1

/* Define to 1 if you have the `getcwd' function. */
#define HAVE_GETCWD 1

/* Define if you have the iconv() function and it works. */
#define HAVE_ICONV 1

/* Define to 1 if the system has the type `__uint128_t'. */
#define HAVE_UINT128_T 1

/* Define to 1 if you have the <locale.h> header file. */
#define HAVE_LOCALE_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Define to 1 if you have the <xlocale.h> header file. */
/* #undef HAVE_XLOCALE_H */

/* Define to 1 if you have the `vsnprintf' function. */
#define HAVE_VSNPRINTF 1

/* Define to 1 if you have the `posix_spawnp' function. */
#define HAVE_POSIX_SPAWNP 1

/* Define to 1 if you have the `posix_memalign' function. */
#define HAVE_POSIX_MEMALIGN 1

/* Define to 1 if you have the `vfork' function. */
#define HAVE_VFORK 1

/* Define to 1 if you have the `mmap' function. */
#define HAVE_MMAP 1

/* Define to 1 if you have the `sigaction' function. */
#define HAVE_SIGACTION 1

/* Define to 1 if you have the `statvfs' function. */
#define HAVE_STATVFS 1

/* Define to 1 if you have the `statvfs64' function. */
#define HAVE_STATVFS64 1

/* For .cpp files, define as const if the declaration of iconv() needs const. */
#define ICONV_CPP_CONST 

/* Define for Mac OSX Framework build */
/* #undef MACOSX_FRAMEWORK */

/* Define to 1 if you have fseek64, ftell64 */
#define UNIX_STDIO_64 1

/* Define to name of 64bit open function */
#define VSI_OPEN64 open64

/* Define to name of 64bit ftruncate function */
#define VSI_FTRUNCATE64 ftruncate64

/* Define to name of 64bit lseek func */
#define VSI_LSEEK64 lseek

/* Define to name of 64bit stat function */
#define VSI_STAT64 stat64

/* Use this file to override settings in instances where you're doing FAT compiles
   on Apple.  It is currently off by default because it doesn't seem to work with
   newish ( XCode >= 3/28/11) XCodes */
/* #include "cpl_config_extras.h" */


/* Define to 1 if you have the _SC_PHYS_PAGES' constant. */
#define HAVE_SC_PHYS_PAGES 1

/* Define to 1 if you have the `sched_getaffinity' function. */
#define HAVE_SCHED_GETAFFINITY 1

/* Define to 1 if you have the `std::shared_mutex' function. */
#define HAVE_SHARED_MUTEX 1

/* Define to 1 if you have the `uselocale' function. */
#define HAVE_USELOCALE 1

/* Define to 1 if libc don't deprecate sprintf */
#ifndef DONT_DEPRECATE_SPRINTF
#define DONT_DEPRECATE_SPRINTF 0
#endif

/* Define to 1 if the compiler supports -Wzero-as-null-pointer-constant */
#define HAVE_GCC_WARNING_ZERO_AS_NULL_POINTER_CONSTANT 1

/* Define if building a static windows lib */
/* #undef CPL_DISABLE_DLL */

/* Define to 1 if you have the <atlbase.h> header file. */
/* #undef HAVE_ATLBASE_H */

/* Define to 1 if you have the `ctime_r' function. */
#define HAVE_CTIME_R 1

/* Define to 1 if you have the `localtime_r' function. */
#define HAVE_LOCALTIME_R 1

/* Define to 1 if you have the `gmtime_r' function. */
#define HAVE_GMTIME_R 1

#endif /* GDAL_COMPILATION */

#endif
